{% raw %}
{% load mote_tags %}
<div class="grid-teaser-wrapper">
  <ul class="grid-teaser-list">
    <li class="grid-teaser-list__item">
      {% render_element element.aspect.basics.article-teaser.grid-teaser-thumbnail %}
      {% render_element element.aspect.basics.article-teaser.grid-teaser-title %}
    </li>
    <li class="grid-teaser-list__item">
      {% render_element element.aspect.basics.article-teaser.grid-teaser-thumbnail %}
      {% render_element element.aspect.basics.article-teaser.grid-teaser-title %}
    </li>
    <li class="grid-teaser-list__item">
      {% render_element element.aspect.basics.article-teaser.grid-teaser-thumbnail %}
      {% render_element element.aspect.basics.article-teaser.grid-teaser-title %}
    </li>
    <li class="grid-teaser-list__item">
      {% render_element element.aspect.basics.article-teaser.grid-teaser-thumbnail %}
      {% render_element element.aspect.basics.article-teaser.grid-teaser-title %}
    </li>
  </ul>
</div>
{% endraw %}
