{% raw %}
{% load mote_tags %}
{% render_element element.aspect.basics.languages.language-list-center %}
  <div id="header" class="header">
    <div class="header__menu">
      {% render_element element.aspect.basics.logo %}
      {% render_element element.aspect.basics.menu %}
    </div>
  </div>
{% endraw %}
