{% load wagtailcore_tags wagtailimages_tags core_tags i18n %}
<ul class="promoted-article-list promoted-article-list--{{type}}">
  {% for article in articles %}
    <li class="promoted-article-list__item promoted-article-list__item-standard">
      {% if article.get_effective_image %}
        <a href="{% pageurl article %}" class="promoted-article-list__anchor">
          {% include "patterns/basics/images/article-thumbnail.html" with curve="none" %}
        </a>
      {% endif %}
      <a href="{% pageurl article %}" class="promoted-article-list__anchor">
        <h3 class="heading promoted-article__title">
          {{article.title}}
        </h3>
      </a>
      <p class="promoted-article-list__desc">{{article.subtitle}}</p>
    </li>
  {% endfor %}
</ul>
