{% load i18n %}
<h1 class="heading section-listing-latest__title">{% trans "Recommended" %}</h1>
{% include "patterns/basics/article-teasers/latest-promoted_variations/latest-articles_standard.html" %}
