header {
  .search-form {
    float: none;
    display: inline-block;
    vertical-align: middle;
    padding: 5px 10px 5px 25px;
    margin: 0;
  }
}
.heading__icon {
  display: inline-block;
}
.download__cta {
  float: right;
}
.filter {
  padding: 10px 0 0;
  &__label {
    padding: 0;
  }
  ul,
  &-list {
    padding: 0;
    display: inline-block;
    &__item,li {
      list-style: none;
    }
  }
  form {
    display: inline-block;
  }
  input {
    padding: 0.5em 1.2em;
  }
}


//Content Wrapper Filter
.content-wrapper .changelist-filter {
  padding: 0;
  display: inline-block;
  width: 100%;
  h2, h3, ul, #date_range_filter {
    padding: 0 15px;
  }
  h2 { line-height: 30px; }
}
//Filter date popup
.xdsoft_datetimepicker {
    z-index: 9999;
}

@media screen and (min-width: 50em) {
  .content-wrapper .changelist-filter {
    ul, #date_range_filter {
      width: auto;
    }
  }
}
@media screen and (min-width: 700px) {
  .content-wrapper .changelist-filter {
    width: auto;
    float: left;
    width: 250px;
    z-index: 9999;
    h2, h3, ul, #date_range_filter {
      margin: 0;
      padding: 0 5px 0 5px;
    }
  }
}
