{% block navigation %}
  {% load wagtailcore_tags core_tags i18n %}

  {% block section_menu %}
  <div class="navigation">
    {% load_sections as sections %}
    <ul class="nav-list nav-list--horizontal nav-list--horizontal-{{class}}">
      {% for section in sections %}
        <li class="nav-list__item nav-list__item--horizontal">
          <a href="{% pageurl section %}" class="nav-list__anchor nav-list__anchor--horizontal">
            {{section.title}}
          </a>
        </li>
      {% endfor %}
    </ul>
  </div>
  {% endblock %}
{% endblock %}
