.breadcrumbs {
  border-buttom: 1px #e7e7e7 solid;
  padding: 10px 0;
  font-size: 14px;
  &-list {
    margin: 0;
    padding: 0 15px;
    list-style: none;
    &__item {
      display: inline-block;
      &:after {
        content: " > ";
        color: #777;
        margin: 0 2px 0 5px;
        font-weight: 700;
      }
      &:last-child:after {
        content: " ";
        display: inline-block;
      }

      &-pipe {
        &:after {
          content: " / ";
        }
      }
    }
    &__anchor {
      color: #777;
      font-weight: 700;
      text-transform: uppercase;
      @media screen and (min-width: 320px) {
        text-decoration: none;
      }
    }
  }
}
