$string: 's';
.tag, .tag#{$string} {
  background: #fff;
  padding: 10px 15px;
  &-list {
      margin: 0;
      padding: 0;
      &__item {
        background: #EBEBEB;
        display: inline-block;
        padding: 5px;
        margin: 5px 0;
        font-size: 13px;
        cursor: pointer;
        line-height: 1.5;
        a, &-anchor {
          color: #252627;
          text-decoration: none;
          word-break: break-word;
        }
      }
  }
  &__name {
    color: #017AF5;
    font-weight: bold;
  }

  &__link {
    display: block;
    color: #909497;
  }

  &__label {
    display: inline-block;
    color: #909497;
    padding: 10px;
    margin: 5px;
    font-size: 14px;
    cursor: pointer;
    line-height: 1.5;
    text-transform: uppercase;
    font-weight: normal;
    &:before {
      content: "";
      display: block;
      color: #000;
      width: 20px;
      background: #000;
      height: 2px;
      margin-bottom: 8px;
    }
  }
}
